@@ -9,9 +9,11 @@
 # terms of the BSD-3 license. We welcome feedback and contributions, see file
 # CONTRIBUTING.md for details.
 
+import io
 import ipywidgets as widgets
 from IPython.display import display as ipydisplay
 from traitlets import Unicode, Int, Bool
+from typing import Union, Tuple
 from ._version import extension_version
 
 try:
@@ -21,19 +23,16 @@ except ImportError:
     Mesh = object
     GridFunction = object
 
+Stream = Union[Tuple[Mesh, GridFunction], Mesh, str]
+
 
 def to_stream(mesh: Mesh, gf: GridFunction = None) -> str:
-    stream = "solution\n" if gf is not None else "mesh\n"
-    temp_file = "temp.saved"
-    mesh.Print(temp_file)
-    with open(temp_file, "r") as f:
-        stream += f.read()
-    if gf is not None:
-        stream += "\n"
-        gf.Save(temp_file)
-        with open(temp_file, "r") as f:
-            stream += f.read()
-    return stream
+    sio = io.StringIO()
+    sio.write("solution\n" if gf is not None else "mesh\n")
+    mesh.WriteToStream(sio)
+    if gf:
+        gf.WriteToStream(sio)
+    return sio.getvalue()
 
 
 @widgets.register
@@ -46,14 +45,14 @@ class glvis(widgets.DOMWidget):
     _view_module = Unicode("glvis-jupyter").tag(sync=True)
     _view_module_version = Unicode("^" + extension_version).tag(sync=True)
 
-    _data_str = Unicode().tag(sync=True)
-    _data_type = Unicode().tag(sync=True)
-    _width = Int().tag(sync=True)
-    _height = Int().tag(sync=True)
-    _is_new_stream = Bool().tag(sync=True)
+    data_str = Unicode().tag(sync=True)
+    data_type = Unicode().tag(sync=True)
+    width = Int().tag(sync=True)
+    height = Int().tag(sync=True)
+    is_new_stream = Bool().tag(sync=True)
 
-    def _sync(self, data, is_new=True):
-        self._is_new_stream = is_new
+    def _sync(self, data: Stream, is_new: bool = True):
+        self.is_new_stream = is_new
         if isinstance(data, str):
             stream = data
         elif isinstance(data, tuple):
@@ -63,25 +62,27 @@ class glvis(widgets.DOMWidget):
         else:
             raise TypeError
         offset = stream.find("\n")
-        self._data_type = stream[0:offset]
-        self._data_str = stream[offset + 1:]
+        self.data_type = stream[0:offset]
+        self.data_str = stream[offset + 1:]
 
-    def __init__(self, data, width=640, height=480, *args, **kwargs):
+    def __init__(
+        self, data: Stream, width: int = 640, height: int = 480, *args, **kwargs
+    ):
         widgets.DOMWidget.__init__(self, *args, **kwargs)
         self.set_size(width, height)
         self._sync(data, is_new=True)
 
-    def display(self, data):
+    def plot(self, data: Stream):
         self._sync(data, is_new=True)
 
-    def update(self, data):
+    def update(self, data: Stream):
         self._sync(data, is_new=False)
 
-    def set_size(self, width, height):
-        self._width = width
-        self._height = height
+    def set_size(self, width: int, height: int):
+        self.width = width
+        self.height = height
 
-    def show(self):
+    def render(self):
         ipydisplay(self)
 
     def serialize(self):
@@ -89,5 +90,8 @@ class glvis(widgets.DOMWidget):
 
         glvis(**other.serialize())
         """
-        return {"data": self._data_type + "\n" + self._data_str,
-                "width": self._width, "height": self._height}
+        return {
+            "data": self.data_type + "\n" + self.data_str,
+            "width": self.width,
+            "height": self.height,
+        }
